   /*
    * Copyright (c) 2012, H. J. Ohlbach. All rights reserved.
    * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
    *
    * This code is free software; you can redistribute it and/or modify it
    * under the terms of the GNU General Public License version 2 only, as
    * published by the Free Software Foundation. 
    *
    * This code is distributed in the hope that it will be useful, but WITHOUT
    * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    * version 2 for more details.
    */

package Java2OWL;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

/** describes accessor groups of Map-type, for example HashMap<Local,String>
 * <p> Typical accessor groups could be
 * <pre>
    HashMap&lt;Locale,String&gt; texts = new HashMap&lt;Locale,String&gt;();
    
    &#64;J2OWLProperty(name = "hasText", setter = "setTexts", adder="addText", remover="removeText", 
            clearer="clearText", getKey="getKey")
    
    public HashMap&lt;Locale,String&gt; getTexts() {return texts;}
    
    public void setTexts(Object texts) {
        assert(texts instanceof HashMap);
        this.texts = (HashMap<Locale,String>)texts;}
    
    public String getText(Locale locale) {return texts.get(locale);}
    
    public String addText(Locale locale, String string) {return texts.put(locale,string);}
    
    public void removeText(Locale locale) {texts.remove(locale);}
    
    public void clearText() {texts.clear();} 
    
    public Locale getKey(String s) {return new Locale(s);}
  </pre>
    
 * @author ohlbach
 */
class PropertyWrapperMappingFunctional extends PropertyWrapperMapping {
    
    PropertyWrapperMappingFunctional(Class jcl, Method getter, J2OManager manager) throws J2OException {
        super(jcl,getter,manager);}
    
    
    /** determines the key and range classes */
    protected boolean determineClasses(StringBuffer errors) {
        currentPWC.isFunctional = true;
        currentPWC.isMapper = true;
        currentPWC.exactLength.put("setter",true);  // exactly one argument
        currentPWC.exactLength.put("adder",false);
        currentPWC.exactLength.put("remover",false);
        currentPWC.exactLength.put("clearer",true); // no argument
        currentPWC.exactLength.put("getKey",true);  // exactly one argument
        Class rangeClass = currentPWC.getter.getReturnType();
        ParameterizedType rangeType = (ParameterizedType)(currentPWC.getter.getGenericReturnType());
        Type[] componentTypes = rangeType.getActualTypeArguments();
        if(componentTypes.length != 2) {errors.append("Illegal Range Type " + rangeType.toString()+"\n"); return false;}
        Class keyClass = Utilities.isClass(componentTypes[0]);
        if(keyClass == null) {errors.append("Illegal Key Class " + keyClass.getName()+"\n"); return false;}
        currentPWC.keyClass = keyClass;
        Class componentClass = Utilities.isClass(componentTypes[1]);
        if(componentClass == null) {errors.append("Illegal Range Class " + rangeClass.getName()+"\n"); return false;}
        currentPWC.rangeClass = componentClass;
        currentPWC.types.put("setter",new Class[][]{new Class[]{rangeClass}});
        currentPWC.types.put("adder",new Class[][]{new Class[]{keyClass,componentClass}});
        currentPWC.types.put("remover",new Class[][]{new Class[]{keyClass}});
        currentPWC.types.put("clearer",new Class[][]{new Class[]{}});
        currentPWC.types.put("getKey",new Class[][]{new Class[]{keyClass}});
        return true;}
    
    
    
}