/*
    * Copyright (c) 2012, H. J. Ohlbach. All rights reserved.
    * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
    *
    * This code is free software; you can redistribute it and/or modify it
    * under the terms of the GNU General Public License version 2 only, as
    * published by the Free Software Foundation. 
    *
    * This code is distributed in the hope that it will be useful, but WITHOUT
    * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    * version 2 for more details.
    */

package Java2OWL;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

/** This class models relational properties generated from getter methods with collection or array return classes.
 *
 * @author ohlbach
 */
class PropertyWrapperRelational extends PropertyWrapper {

    /** constructs a relational property wrapper
     * 
     * @param jcl       the Java class with the getter
     * @param getter    a getter method with return class like List<T> or T[] 
     * @param manager   the top manager
     * @throws J2OException if errors are encountered.
     */
    PropertyWrapperRelational(Class jcl, Method getter, J2OManager manager) throws J2OException {
        super(jcl,getter,manager);
    }
    /** checks the return type of the getter method.
     * 
     * @param errors for appending errors 
     * @return true if the check was okay.
     * <p> Allowed are for example, String[] or List<String> <br/>
     * Not allowed are List<String>[] or List<String[]> <br/>
     * 
     * 
     */
    protected boolean determineClasses(StringBuffer errors) {
        currentPWC.isFunctional = false;
        currentPWC.isMapper = false;
        currentPWC.exactLength.put("setter",true);
        currentPWC.exactLength.put("adder",false);
        currentPWC.exactLength.put("remover",false);
        currentPWC.exactLength.put("clearer",true);
        Method getter = currentPWC.getter;
        Class rangeClass = getter.getReturnType();
        Class componentClass = null;
        if(rangeClass.isArray()) { // arrays of non-composite classes are allowed.
            Type rangeType = getter.getGenericReturnType();
            componentClass = Utilities.isArrayType(rangeClass, rangeType);}
        else {
            ParameterizedType rangeType = (ParameterizedType)(currentPWC.getter.getGenericReturnType());
            Type[] componentTypes = rangeType.getActualTypeArguments();
            if(componentTypes.length != 1) {errors.append("Illegal Range Type " + rangeType.toString()); return false;}
            componentClass = Utilities.isClass(componentTypes[0]);}
        if(componentClass == null) {errors.append("Illegal Range Class " + rangeClass.getName()); return false;}
        currentPWC.rangeClass = componentClass;
        currentPWC.types.put("setter",new Class[][]{new Class[]{rangeClass}});
        currentPWC.types.put("adder",new Class[][]{new Class[]{componentClass}});
        currentPWC.types.put("remover",new Class[][]{new Class[]{componentClass}});
        currentPWC.types.put("clearer",new Class[][]{new Class[]{}});
        return true;}
    
     /**finds adder and clearer methods and checks the other ones.   */
     protected void findAccessors(J2OWLProperty annotation, StringBuffer errors, StringBuffer warnings) {
        findAccessor("setter", annotation.setter(), false, errors, warnings);
        currentPWC.adder = findAccessor("adder", annotation.adder(), true, errors, warnings);
        findAccessor("remover", annotation.remover(), false, errors, warnings);
        currentPWC.clearer = findAccessor("clearer", annotation.clearer(), true, errors, warnings);
        if(!annotation.getKey().isEmpty())  warnings.append("getKey ignored " + annotation.getKey()+"\n");}
    
    
    
}
