/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Java2OWL;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLDataFactory;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;

/**
 *
 * @author ohlbach
 */
public class J2OClassManagerTest {
    static J2OManager manager;
    static J2OOntologyManager omg;
    static OWLOntologyManager om;
    static OWLDataFactory df;
    
    static J2OClassManager clm;
    
    static OWLOntology ontology;
    static OWLClass odead;
    static OWLClass oliving;
    static OWLClass odl;
    static OWLClass oE1;
    static OWLClass oCL1;
    
    public J2OClassManagerTest() {
    }
    
    class JDeadThing {
    @J2OWLProperty(name = "hasAge")
    int alter() {return 0;}
    }
    
    
    
    class JDeadSubthing extends JDeadThing{}
    
    class JLivingThing{ 
        @J2OWLProperty(name = "hasAge")
        int Alter() {return 0;}}
    
     static Class jdead = J2OClassManagerTest.JDeadThing.class;
     static Class jliving = J2OClassManagerTest.JLivingThing.class;
     static Method dage;
     static Method lage;
            
    @BeforeClass
    public static void setUpClass() throws Exception {
 
        manager = new J2OManager("TestManager", J2OManager.Status.COMPILE,true);
        Logging logger = manager.getLogger();
        //logger.activate(true);
        logger.activate(false);
        omg = manager.getOntologyManager();
        om = omg.ontologyManager;
        df = J2OOntologyManager.dataFactory;
        clm = manager.getClassManager();
    
        String name = "test/Java2OWL/BackgroundOntology.owl";
        OWLOntology ontology = omg.loadBackgroundOntology(name);
        name = "test/Java2OWL/ExtensionOntology1.owl";
        ontology = omg.loadExtensionOntology(name);
        omg.setReasoner("HermiT", true, false, 10000);
        
        ArrayList<OWLClass> classes = omg.getOWLClasses("DeadThing");
        odead = classes.iterator().next();
        classes = omg.getOWLClasses("LivingThing");
        oliving = classes.iterator().next();
        classes = omg.getOWLClasses("DeadOrAlive");
        odl = classes.iterator().next();
        
        classes = omg.getOWLClasses("E1");
        oE1 = classes.iterator().next(); 
        classes = omg.getOWLClasses("CL1");
        oCL1 = classes.iterator().next();
        
        dage = jdead.getDeclaredMethod("alter");
        lage = jliving.getDeclaredMethod("Alter");
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }
    
    @Before
    public void setUp() {
        clm.clear();
        clm.addClassCorrespondence(J2OClassManagerTest.JDeadThing.class, odead);
        clm.addClassCorrespondence(J2OClassManagerTest.JLivingThing.class, oliving);
    }
    
    @After
    public void tearDown() {
    }

    /**
     * Test of getName method, of class J2OClassManager.
     */
    @Test
    public void testGetName() {
        System.out.println("getName");
        String result = clm.getName();
        assertEquals("TestManager_ClassManager", result);
    }

    /**
     * Test of addClassCorrespondence method, of class J2OClassManager.
     */
    @Test
    public void testAddClassCorrespondence() {
        System.out.println("addClassCorrespondence");
        clm.clear();
        ClassWrapper mdead = clm.addClassCorrespondence(J2OClassManagerTest.JDeadThing.class, odead);
        assertEquals(J2OClassManagerTest.JDeadThing.class,mdead.getJavaClass());
        assertEquals(odead,mdead.getOWLClass());
        
        ClassWrapper mliving = clm.addClassCorrespondence(J2OClassManagerTest.JLivingThing.class, oliving);
        assertEquals(J2OClassManagerTest.JLivingThing.class,mliving.getJavaClass());
        assertEquals(oliving,mliving.getOWLClass());
          
    }

    /**
     * Test of getClasses method, of class J2OClassManager.
     */
    @Test
    public void testGetClasses() {
        System.out.println("getClasses");
        Set<Class> classes = clm.getClasses();
        assertEquals(true, classes.contains(J2OClassManagerTest.JLivingThing.class));
        assertEquals(true, classes.contains(J2OClassManagerTest.JDeadThing.class));  
    }

    /**
     * Test of isMapped method, of class J2OClassManager.
     */
    @Test
    public void testIsMapped() {
        System.out.println("isMapped");
        assertEquals(true, clm.isMapped(J2OClassManagerTest.JDeadThing.class));
        assertEquals(false, clm.isMapped(J2OClassManagerTest.class));
    }

    /**
     * Test of getCLW method, of class J2OClassManager.
     */
    @Test
    public void testGetCMD() {
        System.out.println("getCMD");
        Class jcl = J2OClassManagerTest.JDeadThing.class;
        ClassWrapper cmd = clm.getCLW(jcl, true, false);
        assertEquals(odead, cmd.getOWLClass());
        assertEquals(jcl, cmd.getJavaClass());
        
        cmd = clm.getCLW(jcl, false, false);
        assertEquals(odead, cmd.getOWLClass());
        assertEquals(jcl, cmd.getJavaClass());
        
        Class jscl = J2OClassManagerTest.JDeadSubthing.class;
        
        cmd = clm.getCLW(jscl, false, true);
        assertEquals(odead, cmd.getOWLClass());
        assertEquals(jcl, cmd.getJavaClass());
        
        cmd = clm.getCLW(jscl, false, true);
        assertEquals(odead, cmd.getOWLClass());
        assertEquals(jcl, cmd.getJavaClass());
        
        cmd = clm.getCLW(jscl, true, true);
        assertEquals(null, cmd);
        
        cmd = clm.getCLW(J2OClassManagerTest.class, false, true);
        assertEquals(null, cmd);
    }

    
    /**
     * Test of getOWLClass method, of class J2OClassManager.
     */
    @Test
    public void testGetOClass() {
        System.out.println("getOClass");
        Class jcl = J2OClassManagerTest.JDeadThing.class;
        OWLClass ocl = clm.getOWLClass(jcl,false,true);
        assertEquals(odead, ocl);
        jcl = J2OClassManagerTest.JDeadSubthing.class;
        ocl = clm.getOWLClass(jcl,false,true);
        assertEquals(odead, ocl);
        ocl = clm.getOWLClass(jcl,true,true);
        assertEquals(null, ocl);   
    }

  
    /**
     * Test of getCLWs method, of class J2OClassManager.
     */
    @Test
    public void testGetCMDs_3args_1() {
        System.out.println("getCMDs");
        Set<ClassWrapper> result = clm.getCLWs(odead);
        assertEquals(1, result.size());
        assertEquals(jdead, result.iterator().next().getJavaClass());
        
        System.out.println("reasoning from E1, no caching");
        result = clm.getCLWs(oE1);
        assertEquals(2, result.size());
        Set<Class> classes = new HashSet<Class>();
        for(ClassWrapper cmd : result) classes.add(cmd.getJavaClass());
        assertEquals(true,classes.contains(jdead));
        assertEquals(true,classes.contains(jliving));
        
        Class jscl = J2OClassManagerTest.JDeadSubthing.class;
        clm.addClassCorrespondence(jscl, odl);
        System.out.println("reasoning from CL1, caching");
        result = clm.getCLWs(oCL1);
        assertEquals(1, result.size());
        assertEquals(jscl,result.iterator().next().getJavaClass());
        
        System.out.println("reasoning from CL1, cached");
        result = clm.getCLWs(oCL1);
        assertEquals(1, result.size());
        assertEquals(jscl,result.iterator().next().getJavaClass());
    }
   

    /**
     * Test of getJavaClasses method, of class J2OClassManager.
     */
    @Test
    public void testGetJClasses() {
        System.out.println("getJClasses");
        Set<Class> classes = clm.getJavaClasses(oE1);
        assertEquals(2, classes.size());
        
        assertEquals(true,classes.contains(jdead));
        assertEquals(true,classes.contains(jliving));
        
    }

   

    /**
     * Test of getCLWs method, of class J2OClassManager.
     */
    @Test
    public void testGetCMDs_3args_2() {
        System.out.println("getCMDs");
        Set<OWLClass> ocls = new HashSet<OWLClass>();
        ocls.add(oE1);
        ocls.add(oCL1);
        Set<ClassWrapper> result = clm.getCLWs(ocls);
        assertEquals(2, result.size());
        Set<Class> classes = new HashSet<Class>();
        for(ClassWrapper cmd : result) classes.add(cmd.getJavaClass());
        assertEquals(true,classes.contains(jdead));
        assertEquals(true,classes.contains(jliving));
        
    }

    /**
     * Test of addMethod method, of class J2OClassManager.
     */
    @Test
    public void testAddMethod() throws Exception {
        System.out.println("addMethod");
        PropertyWrapper dpmd = clm.addMethod(dage.getDeclaringClass(),dage);
        assertEquals(dage,dpmd.getGetter(jdead));
        PropertyWrapper lpmd = clm.addMethod(lage.getDeclaringClass(),lage);
        assertEquals(lpmd,dpmd);
        assertEquals(lage,dpmd.getGetter(jliving));
        
    }

   
}
