/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Java2OWL;

import TestPackage.SimpleStudent;
import java.util.Arrays;
import java.util.Set;
import org.semanticweb.owlapi.expression.ParserException;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLNamedIndividual;

/**
 *
 * @author ohlbach
 */
public class PerformaceTest {
    static J2OManager manager;
    static J2OOntologyManager omg;
    static J2OClassManager clm;
    static J2OIndividualManager img;
    
    static void initialise() {
        manager = new J2OManager("TestManager",J2OManager.Status.RUN,false);
        manager.getLogger().activateInfo(false);
        omg = manager.getOntologyManager();
       
        clm = manager.getClassManager();
        String name = "test/TestPackage/SimpleStudent.owl";
        omg.loadExtensionOntology(name);
        omg.setReasoner("FaCT++", true, false, 100000);
        
        J2OCompiler comp = manager.getCompiler();
        comp.class2OWL(SimpleStudent.class);
        comp.finishTranslation();  
         
        img = manager.getIndvidualManager();
        
        clm.activateSynchronisation(SimpleStudent.class); }
    
    static SimpleStudent[] makeStudents(int n) {
        SimpleStudent[] students = new SimpleStudent[n];
        for(int i = 0; i < n; ++i) {
            int semester = (int)(6*Math.random());
            String name = "Jones_" + i;
            students[i] =  new SimpleStudent(name,semester);}
        return students;
    }
    
    static float[] testAxiomCreation() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 1000;n <= 10000; n += 1000) {
            initialise();
            students = makeStudents(n);
            long t1 = System.currentTimeMillis();
            for(int i = 0; i < n; ++i) 
                img.object2OWL(students[i]);
            
            long t2 = System.currentTimeMillis();
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;
        }
    
    
     static float[] testIndividualCreation() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 1000;n <= 10000; n += 1000) {
            initialise();
            students = makeStudents(n);
            long t1 = System.currentTimeMillis();
            for(int i = 0; i < n; ++i) 
                img.object2OWL(students[i]);
            omg.applyChanges(false);
            
            long t2 = System.currentTimeMillis();
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;}
     
       static float[] testIndividualCreationDirect() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 1000;n <= 10000; n += 1000) {
            initialise();
            students = makeStudents(n);
            long t1 = System.currentTimeMillis();
            for(int i = 0; i < n; ++i) {
                img.object2OWL(students[i]);
                omg.applyChanges(false);}
            
            long t2 = System.currentTimeMillis();
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;}
     
     
     
     static float[] testConsistency() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 1000;n <= 10000; n += 1000) {
            initialise();
            students = makeStudents(n);
            long t1 = System.currentTimeMillis();
            for(int i = 0; i < n; ++i) 
                img.object2OWL(students[i]);
            omg.applyChanges(true);
            
            long t2 = System.currentTimeMillis();
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;
    }
     
     
    static float[] testSynchronise() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 1000;n <= 10000; n += 1000) {
            initialise();
            students = makeStudents(n);
            
            for(int i = 0; i < n; ++i) {
                img.object2OWL(students[i]);}
            omg.applyChanges(false);
            
            long t1 = System.currentTimeMillis();
            for(int i = 0; i < n; ++i) 
                students[i].setSemester(1); 
            omg.applyChanges(false);
            long t2 = System.currentTimeMillis();
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;}
     
      static float[] testInfer() {
         
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 1000;n <= 10000; n += 1000) {
            initialise();
            
            
            //omg.setReasoner("Pellet", true, false, 100000);
            students = makeStudents(n);
            
            for(int i = 0; i < n; ++i) {
                img.object2OWL(students[i]);}
            for(int i = 0; i < n; ++i) 
                students[i].setSemester(1); 
            omg.applyChanges(false);
            
            OWLClassExpression cle = null;
            try {cle = omg.parseClassExpression("Freshman");} catch (ParserException ex) {}
            long t1 = System.currentTimeMillis();
            Set<OWLNamedIndividual> inds = omg.getInstances(cle);
            long t2 = System.currentTimeMillis();
            if(inds != null) System.out.println("LE " + inds.size());
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;}
     
    
       static float[] testInferJava() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 100;n <= 1000; n += 100) {
            initialise();
            students = makeStudents(n);
            
            for(int i = 0; i < n; ++i) {
                img.object2OWL(students[i]);}
            for(int i = 0; i < n; ++i) 
                students[i].setSemester(1); 
            omg.applyChanges(false);
            img.clear();
            OWLClassExpression cle = null;
            try {cle = omg.parseClassExpression("Freshman");} catch (ParserException ex) {}
           
            Set<OWLNamedIndividual> inds = omg.getInstances(cle);
            long t1 = System.currentTimeMillis();
            for(OWLNamedIndividual ind : inds) img.individual2Java(ind);
            long t2 = System.currentTimeMillis();
            System.out.println("LE " + inds.size());
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;}
     
        static float[] testReclassify() {
        SimpleStudent[] students;
        float[] times = new float[10];
        int m = 0;
        for(int n = 100;n <= 1000; n += 100) {
            initialise();
            students = makeStudents(n);
            IndividualWrapper[] indwrps = new IndividualWrapper[students.length];
            
            for(int i = 0; i < n; ++i) {indwrps[i] = img.object2OWL(students[i]);}
            for(int i = 0; i < n; ++i) students[i].setSemester(1); 
            omg.applyChanges(false);
            
            
            long t1 = System.currentTimeMillis();
            for(int i = 0; i < n; ++i) indwrps[i].reclassify(false); 
            long t2 = System.currentTimeMillis();
            float t = (float)(t2-t1)/(float)n;
            times[m] = t; ++m;
            System.out.println("Objects: " + n + " Time " + t + " ms");}
        return times;}
     
    
   
    
    public static void main(String[] args) {
        
        /*
        float[] t1 = testAxiomCreation();
        System.out.println("T1 " + Arrays.toString(t1));
        //T1 [0.17, 0.1895, 0.143, 0.04025, 0.0090, 0.0105, 0.008142857, 0.03175, 0.004888889, 0.0055]
        
        float[] t2 = testIndividualCreation();
        
        System.out.println("T2 " + Arrays.toString(t2));
        //T2 [0.428, 0.3885, 0.20833333, 0.16125, 0.0484, 0.015833333, 0.028142856, 0.010875, 0.082, 0.0302]
        
        
         float[] t4 = testIndividualCreationDirect();
        System.out.println("T4 " + Arrays.toString(t4));
        //T4 [0.025, 0.026, 0.36166668, 0.022, 0.0484, 0.013833334, 0.15485714, 0.05525, 0.014888889, 0.0144]
       
        
        
        float[] t3 = testConsistency();
        System.out.println("T3 " + Arrays.toString(t3));
        //Fact++
        //T3 [0.556, 0.1395, 0.122, 0.0465, 0.0654, 0.012833334, 0.024142858, 0.070625, 0.025555555, 0.0911]
        //Pellet
        //T3 [0.534, 0.1345, 0.13533333, 0.0765, 0.027, 0.064833336, 0.025285713, 0.07775, 0.025, 0.0986]
        // HermiT
        //T3 [0.576, 0.17, 0.107, 0.12075, 0.1114, 0.036666665, 0.012, 0.031375, 0.013555556, 0.092]
        
        
        float[] t5 = testSynchronise();
        System.out.println("T5 " + Arrays.toString(t5));
        //T5 [0.392, 0.071, 0.037, 0.0115, 0.069, 0.0076666665, 0.022857143, 0.113375, 0.0075555556, 0.0075]
   
  
        float[] t6 = testInfer();
        System.out.println("T6 " + Arrays.toString(t6)); 
        //HermiT
        //T6 [15.39, 5.14, 6.72, 10.0325, 13.22, 17.115, 21.144285, 27.78625, 32.461113, 37.561]
        //Fact++ in steps of 1000
        // T6 [0.69, 0.215, 0.26466668, 0.242, 0.27, 0.303, 0.35657144, 0.41825, 0.4188889, 0.4626]
        // Fact++ in steps of 100
        //T6 [0.93, 0.455, 0.16666667, 0.1875, 0.168, 0.16666667, 0.17, 0.22125, 0.17444444, 0.182]
        //Pellet
        //T6 [20.66, 7.57, 6.81, 8.0275, 11.152, 12.295, 13.522857, 15.24875, 18.876667, 22.367]
       
        float[] t7 = testInferJava();
        System.out.println("T7 " + Arrays.toString(t7)); 
        //T7 [0.175, 0.1185, 0.11466666, 0.1295, 0.134, 0.2215, 0.17628571, 0.198375, 0.18844445, 0.2382]
        //Fact++ 1000
        //T7 [0.116, 0.1075, 0.11466666, 0.19175, 0.1332, 0.1555, 0.17857143, 0.177875, 0.22077778, 0.2396]
        //Fact++ 100
        // T7 [0.27, 0.105, 0.083333336, 0.0825, 0.084, 0.083333336, 0.09, 0.18125, 0.07, 0.128]
        // HermiT 100
        //    [0.39, 0.295, 0.22, 0.3425, 0.274, 0.1, 0.09142857, 0.1075, 0.07333333, 0.075]
        */
        float[] t8 = testReclassify();
        System.out.println("T8 " + Arrays.toString(t8)); 
        // Fact++ 1000
        // T8 [0.482, 0.3785, 0.363, 0.44375, 0.4226, 0.52216667, 0.516, 0.58325, 0.7213333, 0.7348]
        // Fact++ 100
        // T8 [1.23, 1.11, 0.44, 0.4275, 0.376, 0.29333332, 0.37571427, 0.285, 0.23666666, 0.296]
        // HermiT 100
        // T8 [18.64, 5.48, 6.66, 9.49, 12.564, 16.221666, 20.197144, 25.73, 30.056667, 35.082]
        // T8 [24.82, 5.635, 6.61, 9.4375, 12.38, 15.815, 20.03, 25.43125, 29.7, 34.448]
        // Pellet
        // T8 [22.13, 7.58, 10.453333, 13.1325, 15.24, 16.356667, 18.964285, 20.045, 25.757778, 29.237]
    
    }
    
}
 